package com.flow.flowable.configure;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.flow.flowable.converter.CopyServiceTaskXMLConverter;
import com.flow.flowable.converter.CustomServiceTaskXMLConverter;
import com.flow.flowable.converter.JumpServiceTaskXMLConverter;
import com.flow.flowable.expression.VariableBetweenExpressionFunction;
import com.flow.flowable.expression.VariableContainsAllExpressionFunction;
import com.flow.flowable.parser.factory.CustomDefaultActivityBehaviorFactory;
import com.flow.flowable.parser.handler.CopyServiceTaskParseHandler;
import com.flow.flowable.parser.handler.JumpServiceTaskParseHandler;
import com.flow.flowable.validator.CustomServiceTaskValidator;
import org.flowable.bpmn.converter.BpmnXMLConverter;
import org.flowable.common.engine.api.delegate.FlowableFunctionDelegate;
import org.flowable.common.engine.impl.cfg.IdGenerator;
import org.flowable.common.engine.impl.persistence.deploy.DeploymentCache;
import org.flowable.engine.impl.persistence.deploy.ProcessDefinitionCacheEntry;
import org.flowable.engine.parse.BpmnParseHandler;
import org.flowable.job.service.impl.asyncexecutor.JobManager;
import org.flowable.spring.SpringProcessEngineConfiguration;
import org.flowable.spring.boot.EngineConfigurationConfigurer;
import org.flowable.task.service.impl.DefaultTaskPostProcessor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.transaction.PlatformTransactionManager;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Configuration
public class FlowableConfigure implements EngineConfigurationConfigurer<SpringProcessEngineConfiguration> {
    @Autowired(required = false)
    private PlatformTransactionManager transactionManager;
    @Autowired(required = false)
    private ObjectMapper objectMapper;
    @Autowired(required = false)
    private IdGenerator idGenerator;
    @Autowired(required = false)
    private DefaultTaskPostProcessor taskPostProcessor;
    @Autowired(required = false)
    private JobManager jobManager;
    @Autowired(required = false)
    private DeploymentCache<ProcessDefinitionCacheEntry> deploymentCache;

    @Override
    public void configure(SpringProcessEngineConfiguration engineConfiguration) {
        List<BpmnParseHandler> bpmnParseHandlers = new ArrayList<>();
        bpmnParseHandlers.add(new CopyServiceTaskParseHandler());
        bpmnParseHandlers.add(new JumpServiceTaskParseHandler());
        engineConfiguration.setPostBpmnParseHandlers(bpmnParseHandlers);
        engineConfiguration.setCustomServiceTaskValidator(new CustomServiceTaskValidator());
        engineConfiguration.setActivityBehaviorFactory(new CustomDefaultActivityBehaviorFactory());
        List<FlowableFunctionDelegate> customFlowableFunctionDelegates = new ArrayList<>();
        customFlowableFunctionDelegates.add(new VariableContainsAllExpressionFunction());
        customFlowableFunctionDelegates.add(new VariableBetweenExpressionFunction());
        engineConfiguration.setCustomFlowableFunctionDelegates(customFlowableFunctionDelegates);
        BpmnXMLConverter.addConverter(new JumpServiceTaskXMLConverter());
        BpmnXMLConverter.addConverter(new CopyServiceTaskXMLConverter());
        BpmnXMLConverter.addConverter(new CustomServiceTaskXMLConverter());
        if (Objects.nonNull(this.objectMapper)) {
            engineConfiguration.setObjectMapper(this.objectMapper);
        }
        if (Objects.nonNull(this.idGenerator)) {
            engineConfiguration.setIdGenerator(this.idGenerator);
        }
        if (Objects.nonNull(this.taskPostProcessor)) {
            engineConfiguration.setTaskPostProcessor(this.taskPostProcessor);
        }
        if (Objects.nonNull(this.transactionManager)) {
            engineConfiguration.setTransactionManager(this.transactionManager);
        }
        if (Objects.nonNull(this.deploymentCache)) {
            engineConfiguration.setProcessDefinitionCache(this.deploymentCache);
            engineConfiguration.setProcessDefinitionCacheLimit(1000);
            // engineConfiguration.setProcessDefinitionDataManager(new CustomProcessDefinitionDataManager(engineConfiguration, this.deploymentCache));
        }
        if (Objects.nonNull(this.jobManager)) {
            engineConfiguration.setAsyncExecutorActivate(true);
            engineConfiguration.setAsyncExecutorMessageQueueMode(true);
            engineConfiguration.setJobManager(this.jobManager);
        }
    }
}
